@@ -302,7 +302,8 @@ long_docs: Dict[str, str] = {
         Note that the above editor selection only applies for editing the definition file,
         but not for any other actions that may be indirectly triggered by git-machete, including editing of rebase TODO list, commit messages etc.
 
-        The definition file can be always accessed and edited directly under the path returned by `git machete file` (currently fixed to <git-directory>/machete).
+        The definition file can be always accessed and edited directly under the path returned by `git machete file`
+        (usually `.git/machete`, unless worktrees or submodules are involved).
     """,
     "file": """
         <b>Usage: git machete file</b>
